"""
Création du fichier csv pour exploitation par la suite
    Prénom, force, vitesse, ruse, famille
"""

import pandas
from random import gauss, seed

# ======================================================================= creer_individus
def creer_individus ( critere:tuple, famille:str, nb_indiv:int ) -> list :
    """
    Critere contient un 3-uplet de 2-uplet : force , vitesse, ruse -> moyenne, dispersion          
    critere -> ((moyenne, dispersion), (moyenne, dispersion), (moyenne, dispersion))
    famille -> nom de la famille : 'expert', 'renfort', 'eclaireur'
    nb_pers -> nombre d'individus à créer
    """
    individus =[ [ 0, 0, 0, ''] for i in range(nb_indiv) ]
    force = [ int(gauss(critere[0][0], critere[0][1])) for i in range(nb_indiv) ]
    vitesse = [ int(gauss(critere[1][0], critere[1][1])) for i in range(nb_indiv) ]
    ruse = [ int(gauss(critere[2][0], critere[2][1])) for i in range(nb_indiv) ]
    for i in range(nb_indiv) :
        individus[i] = ( force[i], vitesse[i], ruse[i], famille )
    return individus

# ======================================================================= creer_trois_familles
def creer_trois_familles ( pren:list )-> tuple :
    """
    Cette fonction détermine trois catégories de personnages selon
    deux critères prépondérents sur les trois mis en jeu :
        - Experts : Force et Ruse
        - Renforts : Force et Vitesse
        - Eclaireurs : Vitesse et Ruse
    Elle associe ensuite un prénom à chaque 3-uplet de chaque catégorie.
    pour créer ainsi les trois familles.
    """
    moy1 = 250 # Critère prépondérent
    disp1 = 50
    moy2 = 100 # Critère secondaire
    disp2 = 25
    
    nb_indiv = len(pren)//3 # répartir les individus en trois familles
    expert, renfort, eclaireur = ( ((moy1,disp1), (moy2,disp2), (moy1,disp1)),
                                   ((moy1,disp1), (moy1,disp1), (moy2,disp2)),
                                   ((moy2,disp2), (moy1,disp1), (moy1,disp1)) )
    # famille des experts
    famille = creer_individus( expert, 'expert', nb_indiv )
    # créer un personnage : 'prénom', force, vitesse, ruse, 'famille'
    experts = [ (pren.pop(0), famille[i]) for i in range(nb_indiv)]

    # famille des renforts
    famille = creer_individus( renfort, 'renfort', nb_indiv )
    # créer un personnage : 'prénom', force, vitesse, ruse, 'famille'
    renforts = [ (pren.pop(0), famille[i]) for i in range(nb_indiv)]

    # famille des éclaireurs
    famille = creer_individus( eclaireur, 'eclaireur', nb_indiv )
    # créer un personnage : 'prénom', force, vitesse, ruse, 'famille'
    eclaireurs = [ (pren.pop(0), famille[i]) for i in range(nb_indiv)]

    return (experts, renforts, eclaireurs)

# ======================================================================= repartir_personnages
def repartir_personnages ( fichier_prenoms:str, fichier_personnages:str ) :
    """
    Entrée :
        fichier_prenoms -> nom du fichier csv qui contient les prénoms :
            => Garçon, Fille
        fichier_personnages -> nom du fichier csv à créer :
            => Prénom, force, vitesse, ruse, famille
    """
    # Lire le fichier des prénoms
    liste_prenoms=pandas.read_csv(fichier_prenoms)
    garcons=liste_prenoms.loc[:,"Garçon"]
    filles=liste_prenoms.loc[:,"Fille"]

    # regrouper alternativement un prénom garçon / fille dans une seule liste
    n = len(garcons)+len(filles) 
    prenoms = [ garcons[i//2] if i%2 else filles[i//2] for i in range(n) ]
    
    # répartir les prénoms en trois familles selon les trois critères
    trois_familles = creer_trois_familles ( prenoms )
    
    # Enregistrer les trois familles dans un fichier
    with open(fichier_personnages, 'w') as personnages :  # Pointeur de fichier
        # Pour chacune des trois familles
        personnages.write( "Prenom,force,vitesse,ruse,famille\n" ) # ecrire
        for famille in trois_familles :
            # enregistrer chaque individu
            for individu in famille:
                ligne = individu[0]+"," # prenom
                ligne += str(individu[1][0])+',' # force
                ligne += str(individu[1][1])+',' # vitesse
                ligne += str(individu[1][2])+',' # ruse
                ligne += individu[1][3]+'\n' # famille
                personnages.write( ligne ) # ecrire

# =================================================================================
# =======================================================================  main

repartir_personnages ( "Prenoms_source.csv", "Mon_Jeu.csv" )
